import {linkedComponent} from '../package';
import {GithubRepository} from '../shapes/GithubRepository';
import React, {useEffect, useState} from 'react';
import {NamedNode} from 'lincd/lib/models';
import {schema} from 'lincd-schema/lib/ontologies/schema';
import {fetchGitGraph} from '../hooks/git-graphql';
import {NodeSet} from 'lincd/lib/collections/NodeSet';
import {UserAvatar} from 'lincd-sioc/lib/components/UserAvatar';

const MAX_NUM_THUMBS: number = 7;
//nameless functional component with named export.
//export const GitRepositoryCollaborators:FunctionalComponent<{accessToken:string},GithubRepository> = linkedComponent(GithubRepository, ({source,accessToken}) => {

//named functional component with named export.
export const GitRepositoryCollaborators = linkedComponent<GithubRepository, {accessToken: string}>(
  GithubRepository,
  function GitRepositoryCollaborators({source, accessToken}) {
    let repository = new GithubRepository(source);
    // let collaborators = repository.collaborators;

    const [collaborators, setCollaborators] = useState<NodeSet>();

    let numLeft, selection;
    if (collaborators) {
      numLeft = collaborators.size - MAX_NUM_THUMBS;
      selection = [...collaborators].slice(0, MAX_NUM_THUMBS);
    }

    //https://github.com/Semantu/lincd.org
    let parts = source ? source.node.value.split('/') : [];
    let repoName = 'lincd';
    let repoOwner = 'semantu';

    useEffect(() => {
      fetchGitGraph(
        `
      query { 
        repository(owner:"${repoOwner}",name:"${repoName}"){
          url 
          collaborators {
            nodes {
              name
              avatarUrl
              login
              resourcePath
            }
          }
        }
      }`,
        accessToken,
      ).then((repositories) => {
        let repositoryResultNode = repositories.first();
        if (repositoryResultNode) {
          let repositoryResult = new GithubRepository(repositoryResultNode);
          console.log(repositoryResultNode === repository.node);
          setCollaborators(repositoryResult.collaborators);
        }
      });
    }, [source]);

    let userClicked = function (collaborator: NamedNode) {
      collaborator.hasProperty(schema.url) &&
        window.open('https://github.com/' + collaborator.getValue(schema.url), '_blank');
    };

    return (
      <div>
        <p>
          <span className="text-grey text-sm">Collaborators</span>{' '}
          <span className="text-white text-md ml-1">{collaborators && collaborators.size}</span>
        </p>

        {selection && (
          <div className="mt-2">
            {selection.map((collaborator: NamedNode) => {
              return (
                <UserAvatar of={collaborator} key={collaborator.toString()} onClick={() => userClicked(collaborator)} />
              );
            })}
            {numLeft > 0 && <span className="text-grey more">+{numLeft} more</span>}
          </div>
        )}
      </div>
    );
  },
);
